@@ -116,6 +116,10 @@ enum ConvTypes {
   CONV_CPP,
   //  driver::COREDUMP
   CONV_COREDUMP,
+  //  driver::GREEN_CONTEXT
+  CONV_GREEN_CONTEXT,
+  //  driver::ERROR_LOG
+  CONV_ERROR_LOG,
   // runtime::DRIVER_INTERACT
   CONV_DRIVER_INTERACT,
   //  driver::PROFILER_DEPRECATED
@@ -244,7 +248,9 @@ enum cudaVersions {
   CUDA_124 = 12040,
   CUDA_125 = 12050,
   CUDA_126 = 12060,
-  CUDA_LATEST = CUDA_126,
+  CUDA_128 = 12080,
+  CUDA_129 = 12090,
+  CUDA_LATEST = CUDA_129,
   CUDNN_10 = 100,
   CUDNN_20 = 200,
   CUDNN_30 = 300,
@@ -301,7 +307,11 @@ enum cudaVersions {
   CUDNN_950 = 950,
   CUDNN_960 = 960,
   CUDNN_970 = 970,
-  CUDNN_LATEST = CUDNN_970,
+  CUDNN_980 = 980,
+  CUDNN_990 = 990,
+  CUDNN_9100 = 9100,
+  CUDNN_9110 = 9110,
+  CUDNN_LATEST = CUDNN_9110,
   CUTENSOR_1010 = 100010,
   CUTENSOR_1100 = 100100,
   CUTENSOR_1200 = 100200,
@@ -321,7 +331,9 @@ enum cudaVersions {
   CUTENSOR_2010 = 200010,
   CUTENSOR_2020 = 200020,
   CUTENSOR_2021 = 200021,
-  CUTENSOR_LATEST = CUTENSOR_2021,
+  CUTENSOR_2109 = 200109,
+  CUTENSOR_2200 = 200200,
+  CUTENSOR_LATEST = CUTENSOR_2200,
 };
 
 enum hipVersions {
@@ -393,7 +405,8 @@ enum hipVersions {
   HIP_6020 = 6020,
   HIP_6030 = 6030,
   HIP_6040 = 6040,
-  HIP_LATEST = HIP_6040,
+  HIP_7000 = 7000,
+  HIP_LATEST = HIP_7000,
 };
 
 struct cudaAPIversions {
